#!/usr/bin/python

# Copyright (c) 2019, Adam Goossens <adam.goossens@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: keycloak_role

short_description: Allows administration of Keycloak roles using Keycloak API

version_added: 3.4.0

description:
  - This module allows you to add, remove or modify Keycloak roles using the Keycloak REST API. It requires access to the
    REST API using OpenID Connect; the user connecting and the client being used must have the requisite access rights. In
    a default Keycloak installation, admin-cli and an admin user would work, as would a separate client definition with the
    scope tailored to your needs and a user having the expected roles.
  - The names of module options are snake_cased versions of the camelCase ones found in the Keycloak API and its documentation
    at U(https://www.keycloak.org/docs-api/8.0/rest-api/index.html).
  - Attributes are multi-valued in the Keycloak API. All attributes are lists of individual values and are returned that way
    by this module. You may pass single values for attributes when calling the module, and this is translated into a list
    suitable for the API.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  action_group:
    version_added: 10.2.0

options:
  state:
    description:
      - State of the role.
      - On V(present), the role is created if it does not yet exist, or updated with the parameters you provide.
      - On V(absent), the role is removed if it exists.
    default: 'present'
    type: str
    choices:
      - present
      - absent

  name:
    type: str
    required: true
    description:
      - Name of the role.
      - This parameter is required.
  description:
    type: str
    description:
      - The role description.
  realm:
    type: str
    description:
      - The Keycloak realm under which this role resides.
    default: 'master'

  client_id:
    type: str
    description:
      - If the role is a client role, the client ID under which it resides.
      - If this parameter is absent, the role is considered a realm role.
  attributes:
    type: dict
    description:
      - A dict of key/value pairs to set as custom attributes for the role.
      - Values may be single values (for example a string) or a list of strings.
  composite:
    description:
      - If V(true), the role is a composition of other realm and/or client role.
    default: false
    type: bool
    version_added: 7.1.0
  composites:
    description:
      - List of roles to include to the composite realm role.
      - If the composite role is a client role, the C(clientId) (not ID of the client) must be specified.
    default: []
    type: list
    elements: dict
    version_added: 7.1.0
    suboptions:
      name:
        description:
          - Name of the role. This can be the name of a REALM role or a client role.
        type: str
        required: true
      client_id:
        description:
          - Client ID if the role is a client role. Do not include this option for a REALM role.
          - Use the client ID you can see in the Keycloak console, not the technical ID of the client.
        type: str
        aliases:
          - clientId
      state:
        description:
          - Create the composite if present, remove it if absent.
        type: str
        choices:
          - present
          - absent
        default: present

extends_documentation_fragment:
  - community.general._keycloak
  - community.general._keycloak.actiongroup_keycloak
  - community.general._attributes

author:
  - Laurent Paumier (@laurpaum)
"""

EXAMPLES = r"""
- name: Create a Keycloak realm role, authentication with credentials
  community.general.keycloak_role:
    name: my-new-kc-role
    realm: MyCustomRealm
    state: present
    auth_client_id: admin-cli
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: USERNAME
    auth_password: PASSWORD
  delegate_to: localhost

- name: Create a Keycloak realm role, authentication with token
  community.general.keycloak_role:
    name: my-new-kc-role
    realm: MyCustomRealm
    state: present
    auth_client_id: admin-cli
    auth_keycloak_url: https://auth.example.com/auth
    token: TOKEN
  delegate_to: localhost

- name: Create a Keycloak client role
  community.general.keycloak_role:
    name: my-new-kc-role
    realm: MyCustomRealm
    client_id: MyClient
    state: present
    auth_client_id: admin-cli
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: USERNAME
    auth_password: PASSWORD
  delegate_to: localhost

- name: Delete a Keycloak role
  community.general.keycloak_role:
    name: my-role-for-deletion
    state: absent
    auth_client_id: admin-cli
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: USERNAME
    auth_password: PASSWORD
  delegate_to: localhost

- name: Create a keycloak role with some custom attributes
  community.general.keycloak_role:
    auth_client_id: admin-cli
    auth_keycloak_url: https://auth.example.com/auth
    auth_realm: master
    auth_username: USERNAME
    auth_password: PASSWORD
    name: my-new-role
    attributes:
      attrib1: value1
      attrib2: value2
      attrib3:
        - with
        - numerous
        - individual
        - list
        - items
  delegate_to: localhost
"""

RETURN = r"""
msg:
  description: Message as to what action was taken.
  returned: always
  type: str
  sample: "Role myrole has been updated"

proposed:
  description: Representation of proposed role.
  returned: always
  type: dict
  sample: {"description": "My updated test description"}

existing:
  description: Representation of existing role.
  returned: always
  type: dict
  sample:
    {
      "attributes": {},
      "clientRole": true,
      "composite": false,
      "containerId": "9f03eb61-a826-4771-a9fd-930e06d2d36a",
      "description": "My client test role",
      "id": "561703dd-0f38-45ff-9a5a-0c978f794547",
      "name": "myrole"
    }

end_state:
  description: Representation of role after module execution (sample is truncated).
  returned: on success
  type: dict
  sample:
    {
      "attributes": {},
      "clientRole": true,
      "composite": false,
      "containerId": "9f03eb61-a826-4771-a9fd-930e06d2d36a",
      "description": "My updated client test role",
      "id": "561703dd-0f38-45ff-9a5a-0c978f794547",
      "name": "myrole"
    }
"""

import copy

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._keycloak import (
    KeycloakAPI,
    KeycloakError,
    camel,
    get_token,
    is_struct_included,
    keycloak_argument_spec,
)


def main():
    """
    Module execution

    :return:
    """
    argument_spec = keycloak_argument_spec()

    composites_spec = dict(
        name=dict(type="str", required=True),
        client_id=dict(type="str", aliases=["clientId"]),
        state=dict(type="str", default="present", choices=["present", "absent"]),
    )

    meta_args = dict(
        state=dict(type="str", default="present", choices=["present", "absent"]),
        name=dict(type="str", required=True),
        description=dict(type="str"),
        realm=dict(type="str", default="master"),
        client_id=dict(type="str"),
        attributes=dict(type="dict"),
        composites=dict(type="list", default=[], options=composites_spec, elements="dict"),
        composite=dict(type="bool", default=False),
    )

    argument_spec.update(meta_args)

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=(
            [["token", "auth_realm", "auth_username", "auth_password", "auth_client_id", "auth_client_secret"]]
        ),
        required_together=([["auth_username", "auth_password"]]),
        required_by={"refresh_token": "auth_realm"},
    )

    result = dict(changed=False, msg="", diff={}, proposed={}, existing={}, end_state={})

    # Obtain access token, initialize API
    try:
        connection_header = get_token(module.params)
    except KeycloakError as e:
        module.fail_json(msg=str(e))

    kc = KeycloakAPI(module, connection_header)

    realm = module.params.get("realm")
    clientid = module.params.get("client_id")
    name = module.params.get("name")
    state = module.params.get("state")

    # attributes in Keycloak have their values returned as lists
    # using the API. attributes is a dict, so we'll transparently convert
    # the values to lists.
    if module.params.get("attributes") is not None:
        for key, val in module.params["attributes"].items():
            module.params["attributes"][key] = [val] if not isinstance(val, list) else val

    # Filter and map the parameters names that apply to the role
    role_params = [
        x
        for x in module.params
        if x not in list(keycloak_argument_spec().keys()) + ["state", "realm", "client_id"]
        and module.params.get(x) is not None
    ]

    # See if it already exists in Keycloak
    if clientid is None:
        before_role = kc.get_realm_role(name, realm)
    else:
        before_role = kc.get_client_role(name, clientid, realm)

    if before_role is None:
        before_role = {}

    # Build a proposed changeset from parameters given to this module
    changeset = {}

    for param in role_params:
        new_param_value = module.params.get(param)
        old_value = before_role[param] if param in before_role else None
        if new_param_value != old_value:
            changeset[camel(param)] = copy.deepcopy(new_param_value)

    # Prepare the desired values using the existing values (non-existence results in a dict that is save to use as a basis)
    desired_role = copy.deepcopy(before_role)
    desired_role.update(changeset)

    result["proposed"] = changeset
    result["existing"] = before_role

    # Cater for when it doesn't exist (an empty dict)
    if not before_role:
        if state == "absent":
            # Do nothing and exit
            if module._diff:
                result["diff"] = dict(before="", after="")
            result["changed"] = False
            result["end_state"] = {}
            result["msg"] = "Role does not exist, doing nothing."
            module.exit_json(**result)

        # Process a creation
        result["changed"] = True

        if name is None:
            module.fail_json(msg="name must be specified when creating a new role")

        if module._diff:
            result["diff"] = dict(before="", after=desired_role)

        if module.check_mode:
            module.exit_json(**result)

        # create it
        if clientid is None:
            kc.create_realm_role(desired_role, realm)
            after_role = kc.get_realm_role(name, realm)
        else:
            kc.create_client_role(desired_role, clientid, realm)
            after_role = kc.get_client_role(name, clientid, realm)

        if after_role["composite"]:
            after_role["composites"] = kc.get_role_composites(rolerep=after_role, clientid=clientid, realm=realm)

        result["end_state"] = after_role

        result["msg"] = f"Role {name} has been created"
        module.exit_json(**result)

    else:
        if state == "present":
            compare_exclude = ["clientId"]
            if (
                "composites" in desired_role
                and isinstance(desired_role["composites"], list)
                and len(desired_role["composites"]) > 0
            ):
                composites = kc.get_role_composites(rolerep=before_role, clientid=clientid, realm=realm)
                before_role["composites"] = []
                for composite in composites:
                    before_composite = {}
                    if composite["clientRole"]:
                        composite_client = kc.get_client_by_id(id=composite["containerId"], realm=realm)
                        before_composite["client_id"] = composite_client["clientId"]
                    else:
                        before_composite["client_id"] = None
                    before_composite["name"] = composite["name"]
                    before_composite["state"] = "present"
                    before_role["composites"].append(before_composite)
            else:
                compare_exclude.append("composites")
            # Process an update
            # no changes
            if is_struct_included(desired_role, before_role, exclude=compare_exclude):
                result["changed"] = False
                result["end_state"] = desired_role
                result["msg"] = f"No changes required to role {name}."
                module.exit_json(**result)

            # doing an update
            result["changed"] = True

            if module._diff:
                result["diff"] = dict(before=before_role, after=desired_role)

            if module.check_mode:
                module.exit_json(**result)

            # do the update
            if clientid is None:
                kc.update_realm_role(desired_role, realm)
                after_role = kc.get_realm_role(name, realm)
            else:
                kc.update_client_role(desired_role, clientid, realm)
                after_role = kc.get_client_role(name, clientid, realm)
            if after_role["composite"]:
                after_role["composites"] = kc.get_role_composites(rolerep=after_role, clientid=clientid, realm=realm)

            result["end_state"] = after_role

            result["msg"] = f"Role {name} has been updated"
            module.exit_json(**result)

        else:
            # Process a deletion (because state was not 'present')
            result["changed"] = True

            if module._diff:
                result["diff"] = dict(before=before_role, after="")

            if module.check_mode:
                module.exit_json(**result)

            # delete it
            if clientid is None:
                kc.delete_realm_role(name, realm)
            else:
                kc.delete_client_role(name, clientid, realm)

            result["end_state"] = {}

            result["msg"] = f"Role {name} has been deleted"

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2022, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: gio_mime
author:
  - "Alexei Znamensky (@russoz)"
short_description: Set default handler for MIME type, for applications using Gnome GIO
version_added: 7.5.0
description:
  - This module allows configuring the default handler for a specific MIME type, to be used by applications built with the
    Gnome GIO API.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
options:
  mime_type:
    description:
      - MIME type for which a default handler is set.
    type: str
    required: true
  handler:
    description:
      - Default handler set for the MIME type.
    type: str
    required: true
notes:
  - This module is a thin wrapper around the C(gio mime) command (and subcommand).
  - See man gio(1) for more details.
seealso:
  - name: C(gio) command manual page
    description: Manual page for the command.
    link: https://man.archlinux.org/man/gio.1
  - name: GIO Documentation
    description: Reference documentation for the GIO API..
    link: https://docs.gtk.org/gio/
"""

EXAMPLES = r"""
- name: Set chrome as the default handler for https
  community.general.gio_mime:
    mime_type: x-scheme-handler/https
    handler: google-chrome.desktop
  register: result
"""

RETURN = r"""
handler:
  description:
    - The handler set as default.
  returned: success
  type: str
  sample: google-chrome.desktop
version:
  description: Version of gio.
  type: str
  returned: always
  sample: "2.80.0"
  version_added: 10.0.0
"""

from ansible_collections.community.general.plugins.module_utils._gio_mime import gio_mime_get, gio_mime_runner
from ansible_collections.community.general.plugins.module_utils._module_helper import ModuleHelper


class GioMime(ModuleHelper):
    output_params = ["handler"]
    module = dict(
        argument_spec=dict(
            mime_type=dict(type="str", required=True),
            handler=dict(type="str", required=True),
        ),
        supports_check_mode=True,
    )

    def __init_module__(self):
        self.runner = gio_mime_runner(self.module, check_rc=True)
        with self.runner("version") as ctx:
            rc, out, err = ctx.run()
            self.vars.version = out.strip()
        self.vars.set_meta(
            "handler", initial_value=gio_mime_get(self.runner, self.vars.mime_type), diff=True, change=True
        )

    def __run__(self):
        check_mode_return = (0, "Module executed in check mode", "")
        if self.vars.has_changed:
            with self.runner.context(
                args_order="mime mime_type handler", check_mode_skip=True, check_mode_return=check_mode_return
            ) as ctx:
                rc, out, err = ctx.run()
                self.vars.stdout = out
                self.vars.stderr = err
                self.vars.set("run_info", ctx.run_info, verbosity=4)


def main():
    GioMime.execute()


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Cisco and/or its affiliates.
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
nxos_bfd_interfaces class
This class creates a command set to bring the current device configuration
to a desired end-state. The command set is based on a comparison of the
current configuration (as dict) and the provided configuration (as dict).
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_diff,
    to_list,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.facts.facts import Facts
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.utils.utils import (
    flatten_dict,
    search_obj_in_list,
)


class Bfd_interfaces(ConfigBase):
    """
    The nxos_bfd_interfaces class
    """

    gather_subset = ["min"]
    gather_network_resources = ["bfd_interfaces"]
    # exclude_params = []

    def __init__(self, module):
        super(Bfd_interfaces, self).__init__(module)

    def get_bfd_interfaces_facts(self, data=None):
        """Get the 'facts' (the current configuration)

        :returns: A list of interface configs and a platform string
        """
        if self.state not in self.ACTION_STATES:
            self.gather_subset = ["!all", "!min"]
        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset,
            self.gather_network_resources,
            data=data,
        )
        bfd_interfaces_facts = facts["ansible_network_resources"].get("bfd_interfaces", [])

        platform = facts.get("ansible_net_platform", "")
        return bfd_interfaces_facts, platform

    def edit_config(self, commands):
        return self._connection.edit_config(commands)

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {"changed": False}
        warnings = list()
        commands = list()

        if self.state in self.ACTION_STATES:
            (
                existing_bfd_interfaces_facts,
                platform,
            ) = self.get_bfd_interfaces_facts()
        else:
            existing_bfd_interfaces_facts, platform = [], ""

        if self.state in self.ACTION_STATES or self.state == "rendered":
            commands.extend(self.set_config(existing_bfd_interfaces_facts, platform))

        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self.edit_config(commands)
            result["changed"] = True

        if self.state in self.ACTION_STATES:
            result["commands"] = commands

        if self.state in self.ACTION_STATES or self.state == "gathered":
            (
                changed_bfd_interfaces_facts,
                platform,
            ) = self.get_bfd_interfaces_facts()

        elif self.state == "rendered":
            result["rendered"] = commands

        elif self.state == "parsed":
            running_config = self._module.params["running_config"]
            if not running_config:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed",
                )
            result["parsed"], platform = self.get_bfd_interfaces_facts(data=running_config)

        if self.state in self.ACTION_STATES:
            result["before"] = existing_bfd_interfaces_facts
            if result["changed"]:
                result["after"] = changed_bfd_interfaces_facts

        elif self.state == "gathered":
            result["gathered"] = changed_bfd_interfaces_facts

        result["warnings"] = warnings
        return result

    def set_config(self, existing_bfd_interfaces_facts, platform):
        """Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        if re.search("N[56]K", platform):
            # Some platforms do not support the 'bfd' interface keyword;
            # remove the 'bfd' key from each want/have interface.
            orig_want = self._module.params["config"]
            want = []
            for w in orig_want:
                del w["bfd"]
                want.append(w)
            orig_have = existing_bfd_interfaces_facts
            have = []
            for h in orig_have:
                del h["bfd"]
                have.append(h)
        else:
            want = self._module.params["config"]
            have = existing_bfd_interfaces_facts

        resp = self.set_state(want, have)
        return to_list(resp)

    def set_state(self, want, have):
        """Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        state = self._module.params["state"]
        if state in ("overridden", "merged", "replaced", "rendered") and not want:
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(state),
            )

        cmds = list()
        if state == "overridden":
            cmds.extend(self._state_overridden(want, have))
        elif state == "deleted":
            cmds.extend(self._state_deleted(want, have))
        else:
            for w in want:
                if state in ["merged", "rendered"]:
                    cmds.extend(self._state_merged(flatten_dict(w), have))
                elif state == "replaced":
                    cmds.extend(self._state_replaced(flatten_dict(w), have))
        return cmds

    def _state_replaced(self, want, have):
        """The command generator when state is replaced

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        cmds = []
        obj_in_have = search_obj_in_list(want["name"], have, "name")
        if obj_in_have:
            diff = dict_diff(want, obj_in_have)
        else:
            diff = want
        merged_cmds = self.set_commands(want, have)
        if "name" not in diff:
            diff["name"] = want["name"]

        replaced_cmds = []
        if obj_in_have:
            replaced_cmds = self.del_attribs(diff)
        if replaced_cmds or merged_cmds:
            for cmd in set(replaced_cmds).intersection(set(merged_cmds)):
                merged_cmds.remove(cmd)
            cmds.extend(replaced_cmds)
            cmds.extend(merged_cmds)
        return cmds

    def _state_overridden(self, want, have):
        """The command generator when state is overridden

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        cmds = []
        for h in have:
            # Clean up bfd attrs for any interfaces not listed in the play
            h = flatten_dict(h)
            obj_in_want = flatten_dict(search_obj_in_list(h["name"], want, "name"))
            if obj_in_want:
                # Let the 'want' loop handle all vals for this interface
                continue
            cmds.extend(self.del_attribs(h))
        for w in want:
            # Update any want attrs if needed. The overridden state considers
            # the play as the source of truth for the entire device, therefore
            # set any unspecified attrs to their default state.
            w = self.set_none_vals_to_defaults(flatten_dict(w))
            cmds.extend(self.set_commands(w, have))
        return cmds

    def _state_merged(self, want, have):
        """The command generator when state is merged

        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """
        return self.set_commands(want, have)

    def _state_deleted(self, want, have):
        """The command generator when state is deleted

        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        if not (want or have):
            return []
        cmds = []
        if want:
            for w in want:
                obj_in_have = flatten_dict(search_obj_in_list(w["name"], have, "name"))
                cmds.extend(self.del_attribs(obj_in_have))
        else:
            for h in have:
                cmds.extend(self.del_attribs(flatten_dict(h)))
        return cmds

    def del_attribs(self, obj):
        if not obj or len(obj.keys()) == 1:
            return []
        cmds = []
        # 'bfd' and 'bfd echo' are enabled by default so the handling is
        # counter-intuitive; we are enabling them to remove them. The end result
        # is that they are removed from the interface config on the device.
        if "bfd" in obj and "disable" in obj["bfd"]:
            cmds.append("bfd")
        if "echo" in obj and "disable" in obj["echo"]:
            cmds.append("bfd echo")
        if cmds:
            cmds.insert(0, "interface " + obj["name"])
        return cmds

    def set_none_vals_to_defaults(self, want):
        # Set dict None values to default states
        if "bfd" in want and want["bfd"] is None:
            want["bfd"] = "enable"
        if "echo" in want and want["echo"] is None:
            want["echo"] = "enable"
        return want

    def diff_of_dicts(self, want, obj_in_have):
        diff = set(want.items()) - set(obj_in_have.items())
        diff = dict(diff)
        if diff and want["name"] == obj_in_have["name"]:
            diff.update({"name": want["name"]})
        return diff

    def add_commands(self, want):
        if not want:
            return []
        cmds = []
        if "bfd" in want and want["bfd"] is not None:
            cmd = "bfd" if want["bfd"] == "enable" else "no bfd"
            cmds.append(cmd)
        if "echo" in want and want["echo"] is not None:
            cmd = "bfd echo" if want["echo"] == "enable" else "no bfd echo"
            cmds.append(cmd)

        if cmds:
            cmds.insert(0, "interface " + want["name"])
        return cmds

    def set_commands(self, want, have):
        cmds = []
        obj_in_have = flatten_dict(search_obj_in_list(want["name"], have, "name"))
        if not obj_in_have:
            cmds = self.add_commands(want)
        else:
            diff = self.diff_of_dicts(want, obj_in_have)
            cmds = self.add_commands(diff)
        return cmds



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The hash_salt filter plugin
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
name: hash_salt
author: Ken Celenza (@itdependsnetworks)
version_added: "1.0.0"
short_description: The hash_salt filter plugin.
description:
  - The filter plugin produces the salt from a hashed password.
  - Using the parameters below - C(password | ansible.netcommon.hash_salt(template.yml))
notes:
  - The filter plugin produces the salt from a hashed password.
options:
  password:
    description:
    - This source data on which hash_salt invokes.
    - For example C(password | ansible.netcommon.hash_salt),
      in this case C(password) represents the hashed password.
    type: str
    required: True
"""

EXAMPLES = r"""
# Using hash_salt

# playbook

- name: Set the facts
  ansible.builtin.set_fact:
    password: "$1$avs$uSTOEMh65ADDBREAKqzvpb9yBMpzd/"

- name: Invoke hash_salt
  ansible.builtin.debug:
    msg: "{{ password | ansible.netcommon.hash_salt() }}"


# Task Output
# -----------
#
# TASK [Set the facts]
# ok: [host] => changed=false
#   ansible_facts:
#     password: $1$avs$uSTOEMh65ADDBREAKqzvpb9yBMpzd/

# TASK [Invoke hash_salt]
# ok: [host] =>
#   msg: avs
"""

from ansible.errors import AnsibleFilterError
from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)

from ansible_collections.ansible.netcommon.plugins.plugin_utils.argspec_filter_utils import (
    convert_to_native,
)
from ansible_collections.ansible.netcommon.plugins.plugin_utils.hash_salt import hash_salt


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment


@pass_environment
def _hash_salt(*args, **kwargs):
    """Extend vlan data"""

    keys = ["password"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    data = convert_to_native(data)
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="hash_salt")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return hash_salt(**updated_data)


class FilterModule(object):
    """hash_salt"""

    def filters(self):
        """a mapping of filter names to functions"""
        return {"hash_salt": _hash_salt}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Akini Ross (@akinross) <akinross@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_tenant_bgp_peer_prefix_policy
short_description: Manage BGP Peer Prefix Policies in Tenant Policy Templates on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage BGP Peer Prefix Policies in Tenant Policy Templates on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Akini Ross (@akinross)
options:
  template:
    description:
    - The name of the tenant template.
    type: str
    aliases: [ tenant_template ]
    required: true
  name:
    description:
    - The name of the BGP Peer Prefix Policy.
    type: str
    aliases: [ bgp_peer_prefix_policy ]
  uuid:
    description:
    - The UUID of the BGP Peer Prefix Policy.
    - This parameter is required when the BGP Peer Prefix Policy O(name) needs to be updated.
    type: str
  description:
    description:
    - The description of the BGP Peer Prefix Policy.
    - Providing an empty string will remove the O(description="") from the BGP Peer Prefix Policy.
    type: str
  action:
    description:
    - The action of the BGP Peer Prefix Policy.
    - The default value is C(reject).
    type: str
    choices: [ log, reject, restart, shutdown ]
  max_number_of_prefixes:
    description:
    - The maximum number of prefixes for the BGP Peer Prefix Policy.
    - The value must be between 1 and 300000.
    - The default value is 20000 prefixes.
    type: int
    aliases: [ max_prefix, max ]
  threshold_percentage:
    description:
    - The threshold percentage of the BGP Peer Prefix Policy.
    - The value must be between 1 and 100.
    - The default value is 75%.
    type: int
    aliases: [ threshold ]
  restart_time:
    description:
    - The restart time of the BGP Peer Prefix Policy in seconds.
    - The value must be between 1 and 65535.
    - The default value is 1 second.
    type: int
    aliases: [ restart ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Tenant template.
seealso:
- module: cisco.mso.ndo_template
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new BGP Peer Prefix Policy object
  cisco.mso.ndo_tenant_bgp_peer_prefix_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    name: bgp_peer_prefix_policy_1
    action: restart
    max_number_of_prefixes: 1000
    threshold_percentage: 50
    restart_time: 60
    state: present
  register: bgp_peer_prefix_policy_1

- name: Update a BGP Peer Prefix Policy object name with UUID
  cisco.mso.ndo_tenant_bgp_peer_prefix_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    name: bgp_peer_prefix_policy_2
    uuid: "{{ bgp_peer_prefix_policy_1.current.uuid }}"
    state: present

- name: Query a BGP Peer Prefix Policy object with name
  cisco.mso.ndo_tenant_bgp_peer_prefix_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    name: bgp_peer_prefix_policy_1
    state: query
  register: query_uuid

- name: Query a BGP Peer Prefix Policy object with UUID
  cisco.mso.ndo_tenant_bgp_peer_prefix_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    uuid: "{{ bgp_peer_prefix_policy_1.current.uuid }}"
    state: query
  register: query_uuid

- name: Query all BGP Peer Prefix Policy objects in a Tenant Template
  cisco.mso.ndo_tenant_bgp_peer_prefix_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    state: query
  register: query_all

- name: Delete a BGP Peer Prefix Policy object with name
  cisco.mso.ndo_tenant_bgp_peer_prefix_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    name: bgp_peer_prefix_policy_1
    state: absent

- name: Delete a BGP Peer Prefix Policy object with UUID
  cisco.mso.ndo_tenant_bgp_peer_prefix_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: tenant_template
    uuid: "{{ bgp_peer_prefix_policy_1.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""

import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True, aliases=["tenant_template"]),
        name=dict(type="str", aliases=["bgp_peer_prefix_policy"]),
        uuid=dict(type="str"),
        description=dict(type="str"),
        action=dict(type="str", choices=["log", "reject", "restart", "shutdown"]),
        max_number_of_prefixes=dict(type="int", aliases=["max_prefix", "max"]),
        threshold_percentage=dict(type="int", aliases=["threshold"]),
        restart_time=dict(type="int", aliases=["restart"]),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    action = module.params.get("action")
    max_number_of_prefixes = module.params.get("max_number_of_prefixes")
    threshold_percentage = module.params.get("threshold_percentage")
    restart_time = module.params.get("restart_time")

    state = module.params.get("state")

    template_object = MSOTemplate(mso, "tenant", template)
    template_object.validate_template("tenantPolicy")

    bgp_peer_prefix_policies = template_object.template.get("tenantPolicyTemplate", {}).get("template", {}).get("bgpPeerPrefixPolicies", [])
    object_description = "BGP Peer Prefix Policy"
    bgp_peer_prefix_policy_attrs_path = None
    match = None

    if state in ["query", "absent"] and bgp_peer_prefix_policies == []:
        mso.exit_json()
    elif state == "query" and not (name or uuid):
        mso.existing = bgp_peer_prefix_policies
    elif bgp_peer_prefix_policies and (name or uuid):
        match = template_object.get_object_by_key_value_pairs(
            object_description, bgp_peer_prefix_policies, [KVPair("uuid", uuid) if uuid else KVPair("name", name)]
        )
        if match:
            bgp_peer_prefix_policy_attrs_path = "/tenantPolicyTemplate/template/bgpPeerPrefixPolicies/{0}".format(match.index)
            mso.existing = mso.previous = copy.deepcopy(match.details)

    ops = []

    if state == "present":
        if uuid and not mso.existing:
            mso.fail_json(msg="{0} with the UUID: '{1}' not found".format(object_description, uuid))

        if mso.existing and match:
            proposed_payload = copy.deepcopy(match.details)

            if name and mso.existing.get("name") != name:
                ops.append(dict(op="replace", path=bgp_peer_prefix_policy_attrs_path + "/name", value=name))
                proposed_payload["name"] = name

            if description is not None and mso.existing.get("description") != description:
                ops.append(dict(op="replace", path=bgp_peer_prefix_policy_attrs_path + "/description", value=description))
                proposed_payload["description"] = description

            if action and mso.existing.get("action") != action:
                ops.append(dict(op="replace", path=bgp_peer_prefix_policy_attrs_path + "/action", value=action))
                proposed_payload["action"] = action

            if max_number_of_prefixes and mso.existing.get("maxPrefixes") != max_number_of_prefixes:
                ops.append(dict(op="replace", path=bgp_peer_prefix_policy_attrs_path + "/maxPrefixes", value=max_number_of_prefixes))
                proposed_payload["maxPrefixes"] = max_number_of_prefixes

            if threshold_percentage and mso.existing.get("threshold") != threshold_percentage:
                ops.append(dict(op="replace", path=bgp_peer_prefix_policy_attrs_path + "/threshold", value=threshold_percentage))
                proposed_payload["threshold"] = threshold_percentage

            if restart_time and mso.existing.get("restartTime") != restart_time:
                ops.append(dict(op="replace", path=bgp_peer_prefix_policy_attrs_path + "/restartTime", value=restart_time))
                proposed_payload["restartTime"] = restart_time

            mso.sanitize(proposed_payload, collate=True)
        else:
            payload = dict(name=name)
            if description:
                payload["description"] = description
            if action:
                payload["action"] = action
            if max_number_of_prefixes:
                payload["maxPrefixes"] = max_number_of_prefixes
            if threshold_percentage:
                payload["threshold"] = threshold_percentage
            if restart_time:
                payload["restartTime"] = restart_time

            mso.sanitize(payload)
            ops.append(dict(op="add", path="/tenantPolicyTemplate/template/bgpPeerPrefixPolicies/-", value=payload))

    elif state == "absent":
        if mso.existing and match:
            ops.append(dict(op="remove", path=bgp_peer_prefix_policy_attrs_path))

    if not module.check_mode and ops:
        response_object = mso.request(template_object.template_path, method="PATCH", data=ops)
        bgp_peer_prefix_policies = response_object.get("tenantPolicyTemplate", {}).get("template", {}).get("bgpPeerPrefixPolicies", [])
        match = template_object.get_object_by_key_value_pairs(
            object_description, bgp_peer_prefix_policies, [KVPair("uuid", uuid) if uuid else KVPair("name", name)]
        )
        if match:
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_igw_info
version_added: 1.0.0
short_description: Gather information about internet gateways in AWS
description:
    - Gather information about internet gateways in AWS.
author: "Nick Aslanidis (@naslanidis)"
options:
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeInternetGateways.html) for possible filters.
    type: dict
    default: {}
  internet_gateway_ids:
    description:
      - Get details of specific Internet Gateway ID. Provide this value as a list.
    type: list
    elements: str
  convert_tags:
    description:
      - Convert tags from boto3 format (list of dictionaries) to the standard dictionary format.
      - Prior to release 4.0.0 this defaulted to V(false).
    default: true
    type: bool
    version_added: 1.3.0
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# # Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Gather information about all Internet Gateways for an account or profile
  amazon.aws.ec2_vpc_igw_info:
    region: ap-southeast-2
    profile: production
  register: igw_info

- name: Gather information about a filtered list of Internet Gateways
  amazon.aws.ec2_vpc_igw_info:
    region: ap-southeast-2
    profile: production
    filters:
      "tag:Name": "igw-123"
  register: igw_info

- name: Gather information about a specific internet gateway by InternetGatewayId
  amazon.aws.ec2_vpc_igw_info:
    region: ap-southeast-2
    profile: production
    internet_gateway_ids: igw-c1231234
  register: igw_info
"""

RETURN = r"""
changed:
    description: True if listing the internet gateways succeeds.
    type: bool
    returned: always
    sample: false
internet_gateways:
    description: The internet gateways for the account.
    returned: always
    type: complex
    contains:
        attachments:
            description: Any VPCs attached to the internet gateway.
            returned: always
            type: complex
            contains:
                state:
                    description: The current state of the attachment.
                    returned: always
                    type: str
                    sample: available
                vpc_id:
                    description: The ID of the VPC.
                    returned: always
                    type: str
                    sample: vpc-02123b67
        internet_gateway_id:
            description: The ID of the internet gateway.
            returned: always
            type: str
            sample: igw-2123634d
        tags:
            description: Any tags assigned to the internet gateway.
            returned: always
            type: dict
            sample:
                tags:
                    "Ansible": "Test"
"""

from typing import Any
from typing import Dict
from typing import List

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_internet_gateways
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def get_internet_gateway_info(internet_gateway: Dict[str, Any], convert_tags: bool) -> Dict[str, Any]:
    if convert_tags:
        tags = boto3_tag_list_to_ansible_dict(internet_gateway["Tags"])
        ignore_list = ["Tags"]
    else:
        tags = internet_gateway["Tags"]
        ignore_list = []
    internet_gateway_info = {
        "InternetGatewayId": internet_gateway["InternetGatewayId"],
        "Attachments": internet_gateway["Attachments"],
        "Tags": tags,
    }

    internet_gateway_info = camel_dict_to_snake_dict(internet_gateway_info, ignore_list=ignore_list)
    return internet_gateway_info


def list_internet_gateways(connection, module: AnsibleAWSModule) -> List[Dict[str, Any]]:
    params = dict()

    params["Filters"] = ansible_dict_to_boto3_filter_list(module.params.get("filters"))
    convert_tags = module.params.get("convert_tags")

    if module.params.get("internet_gateway_ids"):
        params["InternetGatewayIds"] = module.params.get("internet_gateway_ids")

    try:
        all_internet_gateways = describe_internet_gateways(connection, **params)
        if all_internet_gateways is None:
            module.fail_json("InternetGateway not found")
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, "Unable to describe internet gateways")

    return [get_internet_gateway_info(igw, convert_tags) for igw in all_internet_gateways]


def main() -> None:
    argument_spec = dict(
        filters=dict(type="dict", default=dict()),
        internet_gateway_ids=dict(type="list", default=None, elements="str"),
        convert_tags=dict(type="bool", default=True),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    # Validate Requirements
    try:
        connection = module.client("ec2")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    # call your function here
    results = list_internet_gateways(connection, module)

    module.exit_json(internet_gateways=results)


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The iosxr_bgp_address_family config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""


from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.facts import Facts
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.bgp_address_family import (
    Bgp_address_familyTemplate,
)


class Bgp_address_family(ResourceModule):
    """
    The iosxr_bgp_address_family config class
    """

    def __init__(self, module):
        super(Bgp_address_family, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="bgp_address_family",
            tmplt=Bgp_address_familyTemplate(),
        )
        self.parsers = [
            "router",
            "address_family",
            "advertise_best_external",
            "additional_paths",
            "allocate_label",
            "as_path_loopcheck_out_disable",
            "bgp_attribute_download",
            "bgp_bestpath_origin_as_use",
            "bgp_bestpath_origin_as_allow",
            "bgp_client_to_client_reflection_cluster_id",
            "bgp_reflection_disable",
            "bgp_dampening",
            "bgp_label_delay",
            "bgp_import_delay",
            "bgp_origin_as_validation",
            "bgp_scan_time",
            "default_martian_check_disable",
            "distance",
            "dynamic_med",
            "maximum_paths_ibgp",
            "maximum_paths_ebgp",
            "maximum_paths_eibgp",
            "optimal_route_reflection",
            "nexthop",
            "permanent_network_route_policy",
            "retain_local_label",
            "update",
            "global_table_multicast",
            "segmented_multicast",
            "inter_as_install",
            "vrf_all_conf",
            "weight",
            "route_target_download",
            "label_mode",
            "mvpn_single_forwarder_selection_highest_ip_address",
            "mvpn_single_forwarder_selection_all",
            "table_policy",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = self.want
        haved = self.have
        for entry in self.want, self.have:
            self._bgp_list_to_dict(entry)

        # if state is deleted, clean up global params
        if self.state == "deleted":
            if wantd:
                to_del = {"address_family": self._set_to_delete(haved, wantd)}
                haved.update(to_del)

            wantd = {"as_number": haved.get("as_number")}

        else:
            wantd = self.want
            # if state is merged, merge want onto have and then compare
            if self.state == "merged":
                wantd = dict_merge(self.have, self.want)

        self._compare(want=wantd, have=haved)

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Bgp_global network resource.
        """

        self._compare_af(want=want, have=have)
        if self.commands and "router bgp" not in self.commands[0]:
            self.commands.insert(
                0,
                self._tmplt.render(
                    {"as_number": want["as_number"]},
                    "router",
                    False,
                ),
            )

    def _compare_af(self, want, have):
        """Custom handling of afs option
        :params want: the want BGP dictionary
        :params have: the have BGP dictionary
        """
        wafs = want.get("address_family", {})
        hafs = have.get("address_family", {})
        vrf_want_have = []
        for name, entry in wafs.items():
            begin = len(self.commands)
            af_have = hafs.pop(name, {})
            if "vrf" in entry:
                vrf_want_have.append((entry, af_have))

            else:
                self.compare(parsers=self.parsers, want=entry, have=af_have)
                self._compare_lists(want=entry, have=af_have)
                if len(self.commands) != begin:
                    self.commands.insert(
                        begin,
                        self._tmplt.render(
                            {
                                "afi": entry.get("afi"),
                                "safi": entry.get("safi"),
                            },
                            "address_family",
                            False,
                        ),
                    )

        # compare af under vrf separately to ensure correct generation of commands
        for waf, haf in vrf_want_have:
            begin = len(self.commands)
            self.compare(parsers=self.parsers, want=waf, have=haf)
            self._compare_lists(want=waf, have=haf)
            if len(self.commands) != begin:
                self.commands.insert(
                    begin,
                    self._tmplt.render(
                        {"afi": waf.get("afi"), "safi": waf.get("safi")},
                        "address_family",
                        False,
                    ),
                )
                self.commands.insert(
                    begin,
                    self._tmplt.render({"vrf": waf.get("vrf")}, "vrf", False),
                )

        # for deleted and overridden state
        if self.state != "replaced":
            for name, entry in hafs.items():
                if "vrf" in entry:
                    self.addcmd({"vrf": entry.get("vrf")}, "vrf", False)
                self.addcmd(
                    {"afi": entry.get("afi"), "safi": entry.get("safi")},
                    "address_family",
                    True,
                )

    def _compare_lists(self, want, have):
        for attrib in ["aggregate_address", "networks", "redistribute"]:
            wdict = want.get(attrib, {})
            hdict = have.get(attrib, {})
            for key, entry in wdict.items():
                if entry != hdict.pop(key, {}):
                    self.addcmd(entry, attrib.format(attrib), False)

            # remove remaining items in have for replaced
            for entry in hdict.values():
                self.addcmd(entry, attrib.format(attrib), True)

    def _bgp_list_to_dict(self, entry):
        """Convert list of items to dict of items
           for efficient diff calculation.
        :params entry: data dictionary
        """
        for item in entry.get("address_family", []):
            item["aggregate_address"] = {x["value"]: x for x in item.get("aggregate_address", [])}
            item["networks"] = {x["network"]: x for x in item.get("networks", [])}
            item["redistribute"] = {
                (x.get("id"), x["protocol"]): x for x in item.get("redistribute", [])
            }

        if "address_family" in entry:
            entry["address_family"] = {
                "address_family_" + x["afi"] + "_" + x["safi"] + "_vrf_" + x.get("vrf", ""): x
                for x in entry.get("address_family", [])
            }

    def _get_config(self):
        return self._connection.get("show running-config router bgp")

    def _set_to_delete(self, haved, wantd):
        afs_to_del = {}
        h_addrs = haved.get("address_family", {})
        w_addrs = wantd.get("address_family", {})
        for af, h_addr in h_addrs.items():
            if af in w_addrs:
                afs_to_del[af] = h_addr
        return afs_to_del



#!/usr/bin/python

# Copyright (c) 2015, Kevin Brebanov <https://github.com/kbrebanov>
# Based on pacman (Afterburn <https://github.com/afterburn>, Aaron Bull Schaefer <aaron@elasticdog.com>)
# and apt (Matthew Williams <matthew@flowroute.com>) modules.
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: apk
short_description: Manages apk packages
description:
  - Manages C(apk) packages for Alpine Linux.
author: "Kevin Brebanov (@kbrebanov)"
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  available:
    description:
      - During upgrade, reset versioned world dependencies and change logic to prefer replacing or downgrading packages (instead
        of holding them) if the currently installed package is no longer available from any repository.
    type: bool
    default: false
  name:
    description:
      - A package name, like V(foo), or multiple packages, like V(foo,bar).
      - Do not include additional whitespace when specifying multiple packages as a string. Prefer YAML lists over comma-separating
        multiple package names.
    type: list
    elements: str
  no_cache:
    description:
      - Do not use any local cache path.
    type: bool
    default: false
    version_added: 1.0.0
  repository:
    description:
      - A package repository or multiple repositories. Unlike with the underlying apk command, this list overrides the system
        repositories rather than supplement them.
    type: list
    elements: str
  state:
    description:
      - Indicates the desired package(s) state.
      - V(present) ensures the package(s) is/are present. V(installed) can be used as an alias.
      - V(absent) ensures the package(s) is/are absent. V(removed) can be used as an alias.
      - V(latest) ensures the package(s) is/are present and the latest version(s).
    default: present
    choices: ["present", "absent", "latest", "installed", "removed"]
    type: str
  update_cache:
    description:
      - Update repository indexes. Can be run with other steps or on its own.
    type: bool
    default: false
  upgrade:
    description:
      - Upgrade all installed packages to their latest version.
    type: bool
    default: false
  world:
    description:
      - Use a custom world file when checking for explicitly installed packages. The file is used only when a value is provided
        for O(name), and O(state) is set to V(present) or V(latest).
    type: str
    default: /etc/apk/world
    version_added: 5.4.0
notes:
  - O(name) and O(upgrade) are mutually exclusive.
  - When used with a C(loop:) each package is processed individually, it is much more efficient to pass the list directly
    to the O(name) option.
  - This module expects C(apk) to run in non-interactive mode. On Chimera Linux, C(apk) runs interactively by default,
    which will cause this module to hang. Interactive mode can be configured in C(/etc/apk/config). Even though not
    officially supported, other systems using C(apk) should be able to use this module, as long as C(apk) is running
    in non-interactive mode by default.
"""

EXAMPLES = r"""
- name: Update repositories and install foo package
  community.general.apk:
    name: foo
    update_cache: true

- name: Update repositories and install foo and bar packages
  community.general.apk:
    name: foo,bar
    update_cache: true

- name: Remove foo package
  community.general.apk:
    name: foo
    state: absent

- name: Remove foo and bar packages
  community.general.apk:
    name: foo,bar
    state: absent

- name: Install the package foo
  community.general.apk:
    name: foo
    state: present

- name: Install the packages foo and bar
  community.general.apk:
    name: foo,bar
    state: present

- name: Update repositories and update package foo to latest version
  community.general.apk:
    name: foo
    state: latest
    update_cache: true

- name: Update repositories and update packages foo and bar to latest versions
  community.general.apk:
    name: foo,bar
    state: latest
    update_cache: true

- name: Update all installed packages to the latest versions
  community.general.apk:
    upgrade: true

- name: Upgrade / replace / downgrade / uninstall all installed packages to the latest versions available
  community.general.apk:
    available: true
    upgrade: true

- name: Update repositories as a separate step
  community.general.apk:
    update_cache: true

- name: Install package from a specific repository
  community.general.apk:
    name: foo
    state: latest
    update_cache: true
    repository: http://dl-3.alpinelinux.org/alpine/edge/main

- name: Install package without using cache
  community.general.apk:
    name: foo
    state: latest
    no_cache: true

- name: Install package checking a custom world
  community.general.apk:
    name: foo
    state: latest
    world: /etc/apk/world.custom
"""

RETURN = r"""
packages:
  description: A list of packages that have been changed.
  returned: when packages have changed
  type: list
  sample: ["package", "other-package"]
"""

import re

# Import module snippets.
from ansible.module_utils.basic import AnsibleModule


def parse_for_packages(stdout):
    packages = []
    data = stdout.split("\n")
    regex = re.compile(r"^\(\s*\d+/\d+\)\s+\S+\s+(\S+)")
    for l in data:
        p = regex.search(l)
        if p:
            packages.append(p.group(1))
    return packages


def update_package_db(module, exit):
    cmd = APK_PATH + ["update"]
    rc, stdout, stderr = module.run_command(cmd, check_rc=False)
    if rc != 0:
        module.fail_json(msg="could not update package db", stdout=stdout, stderr=stderr)
    elif exit:
        module.exit_json(changed=True, msg="updated repository indexes", stdout=stdout, stderr=stderr)
    else:
        return True


def query_toplevel(module, name, world):
    # world contains a list of top-level packages separated by ' ' or \n
    # packages may contain repository (@) or version (=<>~) separator characters or start with negation !
    regex = re.compile(rf"^{re.escape(name)}([@=<>~].+)?$")
    with open(world) as f:
        content = f.read().split()
        for p in content:
            if regex.search(p):
                return True
    return False


def query_package(module, name):
    cmd = APK_PATH + ["-v", "info", "--installed", name]
    rc, stdout, stderr = module.run_command(cmd, check_rc=False)
    return rc == 0


def query_latest(module, name):
    cmd = APK_PATH + ["version", name]
    rc, stdout, stderr = module.run_command(cmd, check_rc=False)
    search_pattern = rf"({re.escape(name)})-[\d\.\w]+-[\d\w]+\s+(.)\s+[\d\.\w]+-[\d\w]+\s+"
    match = re.search(search_pattern, stdout)
    return not (match and match.group(2) == "<")


def query_virtual(module, name):
    cmd = APK_PATH + ["-v", "info", "--description", name]
    rc, stdout, stderr = module.run_command(cmd, check_rc=False)
    search_pattern = rf"^{re.escape(name)}: virtual meta package"
    return bool(re.search(search_pattern, stdout))


def get_dependencies(module, name):
    cmd = APK_PATH + ["-v", "info", "--depends", name]
    rc, stdout, stderr = module.run_command(cmd, check_rc=False)
    dependencies = stdout.split()
    if len(dependencies) > 1:
        return dependencies[1:]
    else:
        return []


def upgrade_packages(module, available):
    if module.check_mode:
        cmd = APK_PATH + ["upgrade", "--simulate"]
    else:
        cmd = APK_PATH + ["upgrade"]
    if available:
        cmd.append("--available")
    rc, stdout, stderr = module.run_command(cmd, check_rc=False)
    packagelist = parse_for_packages(stdout)
    if rc != 0:
        module.fail_json(msg="failed to upgrade packages", stdout=stdout, stderr=stderr, packages=packagelist)
    if re.search(r"^OK", stdout):
        module.exit_json(
            changed=False, msg="packages already upgraded", stdout=stdout, stderr=stderr, packages=packagelist
        )
    module.exit_json(changed=True, msg="upgraded packages", stdout=stdout, stderr=stderr, packages=packagelist)


def install_packages(module, names, state, world):
    upgrade = False
    to_install = []
    to_upgrade = []
    for name in names:
        # Check if virtual package
        if query_virtual(module, name):
            # Get virtual package dependencies
            dependencies = get_dependencies(module, name)
            for dependency in dependencies:
                if state == "latest" and not query_latest(module, dependency):
                    to_upgrade.append(dependency)
        else:
            if not query_toplevel(module, name, world):
                to_install.append(name)
            elif state == "latest" and not query_latest(module, name):
                to_upgrade.append(name)
    if to_upgrade:
        upgrade = True
    if not to_install and not upgrade:
        module.exit_json(changed=False, msg="package(s) already installed")
    packages = to_install + to_upgrade
    if upgrade:
        if module.check_mode:
            cmd = APK_PATH + ["add", "--upgrade", "--simulate"] + packages
        else:
            cmd = APK_PATH + ["add", "--upgrade"] + packages
    else:
        if module.check_mode:
            cmd = APK_PATH + ["add", "--simulate"] + packages
        else:
            cmd = APK_PATH + ["add"] + packages
    rc, stdout, stderr = module.run_command(cmd, check_rc=False)
    packagelist = parse_for_packages(stdout)
    if rc != 0:
        module.fail_json(msg=f"failed to install {packages}", stdout=stdout, stderr=stderr, packages=packagelist)
    module.exit_json(
        changed=True, msg=f"installed {packages} package(s)", stdout=stdout, stderr=stderr, packages=packagelist
    )


def remove_packages(module, names):
    installed = []
    for name in names:
        if query_package(module, name):
            installed.append(name)
    if not installed:
        module.exit_json(changed=False, msg="package(s) already removed")
    names = installed
    if module.check_mode:
        cmd = APK_PATH + ["del", "--purge", "--simulate"] + names
    else:
        cmd = APK_PATH + ["del", "--purge"] + names
    rc, stdout, stderr = module.run_command(cmd, check_rc=False)
    packagelist = parse_for_packages(stdout)
    # Check to see if packages are still present because of dependencies
    for name in installed:
        if query_package(module, name):
            rc = 1
            break
    if rc != 0:
        module.fail_json(msg=f"failed to remove {names} package(s)", stdout=stdout, stderr=stderr, packages=packagelist)
    module.exit_json(
        changed=True, msg=f"removed {names} package(s)", stdout=stdout, stderr=stderr, packages=packagelist
    )


# ==========================================
# Main control flow.


def main():
    module = AnsibleModule(
        argument_spec=dict(
            state=dict(default="present", choices=["present", "installed", "absent", "removed", "latest"]),
            name=dict(type="list", elements="str"),
            no_cache=dict(default=False, type="bool"),
            repository=dict(type="list", elements="str"),
            update_cache=dict(default=False, type="bool"),
            upgrade=dict(default=False, type="bool"),
            available=dict(default=False, type="bool"),
            world=dict(default="/etc/apk/world", type="str"),
        ),
        required_one_of=[["name", "update_cache", "upgrade"]],
        mutually_exclusive=[["name", "upgrade"]],
        supports_check_mode=True,
    )

    # Set LANG env since we parse stdout
    module.run_command_environ_update = dict(LANGUAGE="C", LC_ALL="C")

    global APK_PATH
    APK_PATH = [module.get_bin_path("apk", required=True)]

    p = module.params

    if p["name"] and any(not name.strip() for name in p["name"]):
        module.fail_json(msg="Package name(s) cannot be empty or whitespace-only")

    if p["no_cache"]:
        APK_PATH.append("--no-cache")

    # add repositories to the APK_PATH
    if p["repository"]:
        for r in p["repository"]:
            APK_PATH.extend(["--repository", r, "--repositories-file", "/dev/null"])

    # normalize the state parameter
    if p["state"] in ["present", "installed"]:
        p["state"] = "present"
    if p["state"] in ["absent", "removed"]:
        p["state"] = "absent"

    if p["update_cache"]:
        update_package_db(module, not p["name"] and not p["upgrade"])

    if p["upgrade"]:
        upgrade_packages(module, p["available"])

    if p["state"] in ["present", "latest"]:
        install_packages(module, p["name"], p["state"], p["world"])
    elif p["state"] == "absent":
        remove_packages(module, p["name"])


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, JR Kerkstra <jrkerkstra@example.org>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: redshift_cross_region_snapshots
version_added: 1.0.0
short_description: Manage Redshift Cross Region Snapshots
description:
  - Manage Redshift Cross Region Snapshots. Supports KMS-Encrypted Snapshots.
  - For more information, see U(https://docs.aws.amazon.com/redshift/latest/mgmt/working-with-snapshots.html#cross-region-snapshot-copy)
author:
  - JR Kerkstra (@captainkerk)
options:
  cluster_name:
    description:
      - The name of the cluster to configure cross-region snapshots for.
    required: true
    aliases: [ "cluster" ]
    type: str
  state:
    description:
      - Create or remove the cross-region snapshot configuration.
    choices: [ "present", "absent" ]
    default: present
    type: str
  region:
    description:
      - "The cluster's region."
    required: true
    aliases: [ "source" ]
    type: str
  destination_region:
    description:
      - The region to copy snapshots to.
    required: true
    aliases: [ "destination" ]
    type: str
  snapshot_copy_grant:
    description:
      - A grant for Amazon Redshift to use a master key in the I(destination_region).
      - See U(http://boto3.readthedocs.io/en/latest/reference/services/redshift.html#Redshift.Client.create_snapshot_copy_grant)
    aliases: [ "copy_grant" ]
    type: str
  snapshot_retention_period:
    description:
      - The number of days to keep cross-region snapshots for.
    required: true
    aliases: [ "retention_period" ]
    type: int
extends_documentation_fragment:
  - amazon.aws.region.modules
  - amazon.aws.common.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: configure cross-region snapshot on cluster `johniscool`
  community.aws.redshift_cross_region_snapshots:
    cluster_name: johniscool
    state: present
    region: us-east-1
    destination_region: us-west-2
    retention_period: 1

- name: configure cross-region snapshot on kms-encrypted cluster
  community.aws.redshift_cross_region_snapshots:
    cluster_name: whatever
    state: present
    region: us-east-1
    destination: us-west-2
    copy_grant: 'my-grant-in-destination'
    retention_period: 10

- name: disable cross-region snapshots, necessary before most cluster modifications (rename, resize)
  community.aws.redshift_cross_region_snapshots:
    cluster_name: whatever
    state: absent
    region: us-east-1
    destination_region: us-west-2
"""

RETURN = r""" # """

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


class SnapshotController(object):
    def __init__(self, client, cluster_name):
        self.client = client
        self.cluster_name = cluster_name

    def get_cluster_snapshot_copy_status(self):
        response = self.client.describe_clusters(ClusterIdentifier=self.cluster_name)
        return response["Clusters"][0].get("ClusterSnapshotCopyStatus")

    def enable_snapshot_copy(self, destination_region, grant_name, retention_period):
        if grant_name:
            self.client.enable_snapshot_copy(
                ClusterIdentifier=self.cluster_name,
                DestinationRegion=destination_region,
                RetentionPeriod=retention_period,
                SnapshotCopyGrantName=grant_name,
            )
        else:
            self.client.enable_snapshot_copy(
                ClusterIdentifier=self.cluster_name,
                DestinationRegion=destination_region,
                RetentionPeriod=retention_period,
            )

    def disable_snapshot_copy(self):
        self.client.disable_snapshot_copy(ClusterIdentifier=self.cluster_name)

    def modify_snapshot_copy_retention_period(self, retention_period):
        self.client.modify_snapshot_copy_retention_period(
            ClusterIdentifier=self.cluster_name, RetentionPeriod=retention_period
        )


def requesting_unsupported_modifications(actual, requested):
    if (
        actual["SnapshotCopyGrantName"] != requested["snapshot_copy_grant"]
        or actual["DestinationRegion"] != requested["destination_region"]
    ):
        return True
    return False


def needs_update(actual, requested):
    if actual["RetentionPeriod"] != requested["snapshot_retention_period"]:
        return True
    return False


def run_module():
    argument_spec = dict(
        cluster_name=dict(type="str", required=True, aliases=["cluster"]),
        state=dict(type="str", choices=["present", "absent"], default="present"),
        region=dict(type="str", required=True, aliases=["source"]),
        destination_region=dict(type="str", required=True, aliases=["destination"]),
        snapshot_copy_grant=dict(type="str", aliases=["copy_grant"]),
        snapshot_retention_period=dict(type="int", required=True, aliases=["retention_period"]),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    result = dict(
        changed=False,
        message="",
    )
    connection = module.client("redshift")

    snapshot_controller = SnapshotController(client=connection, cluster_name=module.params.get("cluster_name"))

    current_config = snapshot_controller.get_cluster_snapshot_copy_status()
    if current_config is not None:
        if module.params.get("state") == "present":
            if requesting_unsupported_modifications(current_config, module.params):
                message = (
                    "Cannot modify destination_region or grant_name. Please disable cross-region snapshots, and re-run."
                )
                module.fail_json(msg=message, **result)
            if needs_update(current_config, module.params):
                result["changed"] = True
                if not module.check_mode:
                    snapshot_controller.modify_snapshot_copy_retention_period(
                        module.params.get("snapshot_retention_period")
                    )
        else:
            result["changed"] = True
            if not module.check_mode:
                snapshot_controller.disable_snapshot_copy()
    else:
        if module.params.get("state") == "present":
            result["changed"] = True
            if not module.check_mode:
                snapshot_controller.enable_snapshot_copy(
                    module.params.get("destination_region"),
                    module.params.get("snapshot_copy_grant"),
                    module.params.get("snapshot_retention_period"),
                )
    module.exit_json(**result)


def main():
    run_module()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 Haiyuan Zhang, <haiyzhan@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_adpassword

version_added: "0.2.0"

short_description: Manage application password

description:
    - Manage application password.

options:
    app_id:
        description:
            - The application ID.
        type: str
    service_principal_object_id:
        description:
            - The service principal object ID.
        type: str
    key_id:
        description:
            - The password key ID.
            - It isn't supported anymore in the create operation. See the Azure documentation for more information
              U(https://learn.microsoft.com/en-us/graph/api/application-addpassword?view=graph-rest-1.0&tabs=http#request-body).
        type: str
    end_date:
        description:
            - Date or datemtime after which credentials expire.
            - Default value is one year after current time.
        type: str
    display_name:
        description:
            - The friendly name of the application password.
        type: str
    app_object_id:
        description:
            - The application object ID.
        type: str
    state:
        description:
            - Assert the state of Active Dirctory Password.
            - Use C(present) to create or update a Password and use C(absent) to delete.
            - Update is not supported, if I(state=absent) and I(key_id=None), then all passwords of the application will be deleted.
        default: present
        choices:
            - absent
            - present
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    haiyuan_zhang (@haiyuazhang)
    Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: create ad password
  azure_rm_adpassword:
    app_id: "{{ app_id }}"
    state: present
    display_name: "Password friendly name"
'''

RETURN = '''
end_date:
    description:
        - Date or datemtime after which credentials expire.
        - Default value is one year after current time.
    type: str
    returned: always
    sample: "2021-06-28T06:00:32.637070+00:00"
key_id:
    description:
        - The password key ID
    type: str
    returned: always
    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
start_date:
    description:
        - Date or datetime at which credentials become valid.
        - Default value is current time.
    type: str
    returned: always
    sample: "2020-06-28T06:00:32.637070+00:00"
secret_text:
    description:
        - The application password value.
        - API only returns the application password value at creation.
    type: str
    returned: created
    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
import datetime

try:
    import asyncio
    from msgraph.generated.models.password_credential import PasswordCredential
    from msgraph.generated.applications.item.add_password.add_password_post_request_body import \
        AddPasswordPostRequestBody
    from msgraph.generated.applications.item.remove_password.remove_password_post_request_body import \
        RemovePasswordPostRequestBody
    from msgraph.generated.applications.applications_request_builder import ApplicationsRequestBuilder
    from dateutil.relativedelta import relativedelta
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMADPassword(AzureRMModuleBase):
    def __init__(self):

        self.module_arg_spec = dict(
            app_id=dict(type='str'),
            service_principal_object_id=dict(type='str'),
            app_object_id=dict(type='str'),
            key_id=dict(type='str'),
            display_name=dict(type='str'),
            end_date=dict(type='str'),
            state=dict(type='str', default='present', choices=['present', 'absent']),
        )

        self.state = None
        self.app_id = None
        self.service_principal_object_id = None
        self.app_object_id = None
        self.key_id = None
        self.display_name = None
        self.end_date = None
        self.results = dict(changed=False)

        self.client = None

        super(AzureRMADPassword, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                supports_check_mode=False,
                                                supports_tags=False,
                                                is_ad_resource=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        self._client = self.get_msgraph_client()
        self.resolve_app_obj_id()
        passwords = self.get_all_passwords()

        if self.state == 'present':
            if self.key_id and self.key_exists(passwords):
                self.update_password()
            else:
                self.create_password()
        else:
            if self.key_id is None:
                self.delete_all_passwords(passwords)
            else:
                self.delete_password(passwords)

        return self.results

    def key_exists(self, old_passwords):
        for pd in old_passwords:
            if str(pd.key_id) == self.key_id:
                return True
        return False

    def resolve_app_obj_id(self):
        try:
            if self.app_object_id is not None:
                return
            elif self.app_id or self.service_principal_object_id:
                if not self.app_id:
                    sp = asyncio.get_event_loop().run_until_complete(self.get_service_principal())
                    self.app_id = sp.app_id
                if not self.app_id:
                    self.fail("can't resolve app via service principal object id {0}".format(
                        self.service_principal_object_id))

                apps = asyncio.get_event_loop().run_until_complete(self.get_applications())
                result = list(apps.value)
                if result:
                    self.app_object_id = result[0].id
                else:
                    self.fail("can't resolve app via app id {0}".format(self.app_id))
            else:
                self.fail("one of the [app_id, app_object_id, service_principal_id] must be set")

        except Exception as ge:
            self.fail("error in resolve app_object_id {0}".format(str(ge)))

    def get_all_passwords(self):

        try:
            application = asyncio.get_event_loop().run_until_complete(self.get_application())
            passwordCredentials = application.password_credentials
            return passwordCredentials
        except Exception as ge:
            self.fail("failed to fetch passwords for app {0}: {1}".format(self.app_object_id, str(ge)))

    def delete_all_passwords(self, old_passwords):

        if len(old_passwords) == 0:
            self.results['changed'] = False
            return
        try:
            for pd in old_passwords:
                asyncio.get_event_loop().run_until_complete(self.remove_password(pd.key_id))
            self.results['changed'] = True
        except Exception as ge:
            self.fail("fail to purge all passwords for app: {0} - {1}".format(self.app_object_id, str(ge)))

    def delete_password(self, old_passwords):
        if not self.key_exists(old_passwords):
            self.results['changed'] = False
            return

        num_of_passwords_before_delete = len(old_passwords)

        for pd in old_passwords:
            if str(pd.key_id) == self.key_id:
                try:
                    asyncio.get_event_loop().run_until_complete(self.remove_password(pd.key_id))

                    num_of_passwords_after_delete = len(self.get_all_passwords())
                    if num_of_passwords_after_delete != num_of_passwords_before_delete:
                        self.results['changed'] = True
                except Exception as ge:
                    self.fail("failed to delete password with key id {0} - {1}".format(self.app_id, str(ge)))
                break

    def create_password(self):
        start_date = datetime.datetime.now(datetime.timezone.utc)
        end_date = self.end_date or start_date + relativedelta(years=1)
        display_name = self.display_name

        try:
            request_body = AddPasswordPostRequestBody(
                password_credential=PasswordCredential(
                    start_date_time=start_date,
                    end_date_time=end_date,
                    display_name=display_name
                ),
            )
            pd = asyncio.get_event_loop().run_until_complete(self.add_password(request_body))

            self.results['changed'] = True
            self.results.update(self.to_dict(pd))

        except Exception as ge:
            self.fail("failed to create new password: {0}".format(str(ge)))

    def update_password(self):
        self.fail("update existing password is not supported")

    def to_dict(self, pd):
        return dict(
            end_date=str(pd.end_date_time),
            start_date=str(pd.start_date_time),
            key_id=str(pd.key_id),
            secret_text=str(pd.secret_text)
        )

    async def get_service_principal(self):
        return await self._client.service_principals.by_service_principal_id(self.service_principal_object_id).get()

    async def get_applications(self):
        request_configuration = ApplicationsRequestBuilder.ApplicationsRequestBuilderGetRequestConfiguration(
            query_parameters=ApplicationsRequestBuilder.ApplicationsRequestBuilderGetQueryParameters(
                filter="appId eq '{0}'".format(self.app_id),
            ),
        )
        return await self._client.applications.get(request_configuration=request_configuration)

    async def get_application(self):
        return await self._client.applications.by_application_id(self.app_object_id).get()

    async def remove_password(self, key_id):
        request_body = RemovePasswordPostRequestBody(
            key_id=key_id,
        )
        return await self._client.applications.by_application_id(self.app_object_id).remove_password.post(
            body=request_body)

    async def add_password(self, request_body):
        return await self._client.applications.by_application_id(self.app_object_id).add_password.post(
            body=request_body)


def main():
    AzureRMADPassword()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Bgp_global parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_router_bgp_cmd(config_data):
    command = "router bgp {as_number}".format(**config_data)
    return command


def _tmplt_bgp_address_family(config_data):
    command = ""
    if config_data.get("vrf"):
        command = "vrf {vrf}\n".format(**config_data)
    command += "address-family {afi}".format(**config_data)
    if config_data.get("safi"):
        command += " {safi}".format(**config_data)
    return command


def _tmplt_bgp_params(config_data):
    command = "bgp"
    if config_data["bgp_params"].get("additional_paths"):
        command += " additional-paths {additional_paths}".format(**config_data["bgp_params"])
        if config_data["bgp_params"]["additional_paths"] == "send":
            command += " any"
    elif config_data["bgp_params"].get("next_hop_address_family"):
        command += " next-hop address-family ipv6"
    elif config_data["bgp_params"].get("next_hop_unchanged"):
        command += " next-hop-unchanged"
    elif config_data["bgp_params"].get("redistribute_internal"):
        command += " redistribute-internal"
    elif config_data["bgp_params"].get("route"):
        command += " route install-map {route}".format(**config_data["bgp_params"])
    return command


def _tmplt_bgp_graceful_restart(config_data):
    command = "graceful-restart"
    return command


def _tmplt_bgp_neighbor(config_data):
    command = "neighbor {peer}".format(**config_data["neighbor"])
    if config_data["neighbor"].get("additional_paths"):
        command += " additional-paths {additional_paths}".format(**config_data["neighbor"])
        if config_data["neighbor"]["additional_paths"] == "send":
            command += "any"
    elif config_data["neighbor"].get("activate"):
        command += " activate"
    elif config_data["neighbor"].get("default_originate"):
        command += " default-originate"
        if config_data["neighbor"]["default_originate"].get("route_map"):
            command += " route-map " + config_data["neighbor"]["default_originate"]["route_map"]
        if config_data["neighbor"]["default_originate"].get("always"):
            command += " always"
    elif config_data["neighbor"].get("graceful_restart"):
        command += " graceful-restart"
    elif config_data["neighbor"].get("next_hop_unchanged"):
        command += " next-hop-unchanged"
    elif config_data["neighbor"].get("next_hop_address_family"):
        command += " next-hop addres-family ipv6"
    elif config_data["neighbor"].get("prefix_list"):
        command += " prefix-list {name} {direction}".format(
            **config_data["neighbor"]["prefix_list"],
        )
    elif config_data["neighbor"].get("route_map"):
        command += " route-map {name} {direction}".format(**config_data["neighbor"]["route_map"])
    elif config_data["neighbor"].get("weight"):
        command += " weight {weight}".format(**config_data["neighbor"])
    elif config_data["neighbor"].get("encapsulation"):
        command += " encapsulation {transport}".format(**config_data["neighbor"])
        if config_data["neighbor"]["encapsulation"].get("source_interface"):
            command += " next-hop-self source-interface {source_interface}".format(
                **config_data["neighbor"],
            )
    return command


def _tmplt_bgp_network(config_data):
    command = "network {address}".format(**config_data)
    if config_data.get("route_map"):
        command += " route-map {route_map}".format(**config_data)
    return command


def _tmplt_bgp_redistribute(config_data):
    command = "redistribute {protocol}".format(**config_data)
    if config_data.get("isis_level"):
        command += " {isis_level}".format(**config_data)
    if config_data.get("ospf_route"):
        command += " match {ospf_route}".format(**config_data)
    if config_data.get("route_map"):
        command += " route-map {route_map}".format(**config_data)
    return command


def _tmplt_bgp_route_target(config_data):
    command = "route-target {action}".format(**config_data["route_target"])
    if config_data["route_target"].get("type"):
        command += " {type}".format(**config_data["route_target"])
    if config_data["route_target"].get("route_map"):
        command += " {route_map}".format(**config_data["route_target"])
    if config_data["route_target"].get("target"):
        command += " {target}".format(**config_data["route_target"])
    return command


class Bgp_afTemplate(NetworkTemplate):
    def __init__(self, lines=None):
        super(Bgp_afTemplate, self).__init__(lines=lines, tmplt=self)

    # fmt: off
    PARSERS = [
        {
            "name": "router",
            "getval": re.compile(
                r"""
                ^router\s
                bgp
                \s(?P<as_num>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_router_bgp_cmd,
            "compval": "as_number",
            "result": {"as_number": "{{ as_num }}"},
            "shared": True,
        },
        {
            "name": "address_family",
            "getval": re.compile(
                r"""
                \s*(?P<vrf>vrf\s\S+)*
                \s*address-family
                \s(?P<afi>ipv4|ipv6|evpn)
                \s*(?P<type>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_address_family,
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "afi": "{{ afi }}",
                        "safi": "{{ type }}",
                        "vrf": "{{ vrf.split(" ")[1] }}",
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "bgp_params_additional_paths",
            "getval": re.compile(
                r"""
                \s*bgp
                \s+additional-paths
                \s+(?P<action>\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_params,
            "compval": "bgp_params.additional_paths",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "bgp_params": {
                            "additional_paths": "{{ action }}",
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_params.nexthop_address_family",
            "getval": re.compile(
                r"""
                \s*bgp
                \s+next-hop
                \s+address-family
                \s+ipv6
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_params,
            "compval": "bgp_params.next_hop_address_family",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "bgp_params": {
                            "next_hop_unchanged": "{{ 'ipv6' }}",
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_params.nexthop_unchanged",
            "getval": re.compile(
                r"""
                \s*bgp
                \s+next-hop-unchanged
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_params,
            "compval": "bgp_params.next_hop_unchanged",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "bgp_params": {
                            "next_hop_unchanged": "{{ True }}",
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_params.redistribute_internal",
            "getval": re.compile(
                r"""
                \s*bgp
                \s+redistribute-internal
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_params,
            "compval": "bgp_params.redistribute_internal",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "bgp_params": {
                            "redistribute_internal": "{{ True }}",
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_params.route",
            "getval": re.compile(
                r"""
                \s*bgp
                \s+route
                \s+install-map
                \s+(?P<route>\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_params,
            "compval": "bgp_params.route",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "bgp_params": {
                            "route": "{{ route }}",
                        },
                    },
                },
            },
        },
        {
            "name": "graceful_restart",
            "getval": re.compile(
                r"""
                \s*graceful-restart
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_graceful_restart,
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "graceful_restart": "{{ True }}",
                    },
                },
            },
        },
        {
            "name": "neighbor.activate",
            "getval": re.compile(
                r"""
                \s*neighbor
                \s+(?P<peer>\S+)
                \s+activate
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_neighbor,
            "compval": "neighbor.activate",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "neighbor": {
                            "{{ peer }}": {
                                "peer": "{{ peer }}",
                                "activate": "{{ True }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor.additional_paths",
            "getval": re.compile(
                r"""
                \s*neighbor
                \s+(?P<peer>\S+)
                \s+additional-paths
                \s+(?P<action>\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_neighbor,
            "compval": "neighbor.additional_paths",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "neighbor": {
                            "{{ peer }}": {
                                "peer": "{{ peer }}",
                                "additional_paths": "{{ action }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor.default_originate",
            "getval": re.compile(
                r"""
                \s*neighbor
                \s+(?P<peer>\S+)
                \s+default-originate
                \s*(?P<route_map>route-map\s\S+)*
                \s*(?P<always>always)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_neighbor,
            "compval": "neighbor.default_originate",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "neighbor": {
                            "{{ peer }}": {
                                "peer": "{{ peer }}",
                                "default_originate": {
                                    "route_map": "{{ route_map.split(" ")[1] }}",
                                    "always": "{{ True if always is defined }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor.graceful_restart",
            "getval": re.compile(
                r"""
                \s*neighbor
                \s+(?P<peer>\S+)
                \s+graceful-restart
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_neighbor,
            "compval": "neighbor.graceful_restart",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "neighbor": {
                            "{{ peer }}": {
                                "peer": "{{ peer }}",
                                "graceful_restart": "{{ True }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor.next_hop_unchanged",
            "getval": re.compile(
                r"""
                \s*neighbor
                \s+(?P<peer>\S+)
                \s+next-hop-unchanged
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_neighbor,
            "compval": "neighbor.next_hop_unchanged",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "neighbor": {
                            "{{ peer }}": {
                                "peer": "{{ peer }}",
                                "next_hop_unchanged": "{{ True }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor.next_hop_address_family",
            "getval": re.compile(
                r"""
                \s*neighbor
                \s+(?P<peer>\S+)
                \s+next-hop
                \s+address-family
                \s+ipv6
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_neighbor,
            "compval": "neighbor.next_hop_address_family",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "neighbor": {
                            "{{ peer }}": {
                                "peer": "{{ peer }}",
                                "next_hop_address_family": "{{ 'ipv6' }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor.prefix_list",
            "getval": re.compile(
                r"""
                \s*neighbor
                \s+(?P<peer>\S+)
                \s+prefix-list
                \s+(?P<name>\S+)
                \s+(?P<dir>in|out)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_neighbor,
            "compval": "neighbor.prefix_list",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "neighbor": {
                            "{{ peer }}": {
                                "peer": "{{ peer }}",
                                "prefix_list": {
                                    "name": "{{ name }}",
                                    "direction": "{{ dir }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor.route_map",
            "getval": re.compile(
                r"""
                \s*neighbor
                \s+(?P<peer>\S+)
                \s+route-map
                \s+(?P<name>\S+)
                \s+(?P<dir>in|out)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_neighbor,
            "compval": "neighbor.route_map",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "neighbor": {
                            "{{ peer }}": {
                                "peer": "{{ peer }}",
                                "route_map": {
                                    "name": "{{ name }}",
                                    "direction": "{{ dir }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor.weight",
            "getval": re.compile(
                r"""
                \s*neighbor
                \s+(?P<peer>\S+)
                \s+weight
                \s+(?P<weight>\d+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_neighbor,
            "compval": "neighbor.weight",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "neighbor": {
                            "{{ peer }}": {
                                "peer": "{{ peer }}",
                                "weight": "{{ weight }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "neighbor.encapsulation",
            "getval": re.compile(
                r"""
                \s*neighbor
                \s+default
                \s+encapsulation
                \s+(?P<type>mpls|vxlan)
                \s*(next-hop-self)*
                \s*(source-interface)*
                \s*(?P<interface>\S+\s\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_neighbor,
            "compval": "neighbor.encapsulation",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "neighbor": {
                            "{{ peer }}": {
                                "peer": "{{ peer }}",
                                "encapsulation": {
                                    "transport": "{{ type }}",
                                    "source_interface": "{{ interface }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "network",
            "getval": re.compile(
                r"""
                \s*network
                \s+(?P<address>\S+)
                \s*(route-map)*
                \s*(?P<route_map>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_network,
            "compval": "network",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "network": {
                            "{{ address }}": {
                                "address": "{{ address }}",
                                "route_map": "{{ route_map }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "redistribute",
            "getval": re.compile(
                r"""
                \s*redistribute
                \s+(?P<route>\S+)
                \s*(?P<level>level-1|level-2|level-1-2)*
                \s*(?P<match>match\s\S+)*
                \s*(?P<route_map>route-map\s\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_redistribute,
            "compval": "redistribute",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "redistribute": [
                            {
                                "protocol": "{{ route }}",
                                "route_map": "{{ route_map.split(" ")[1] }}",
                                "isis_level": "{{ level }}",
                                "ospf_route": "{{ match.split(" ")[1] }}",
                            },
                        ],
                    },
                },
            },
        },
        {
            "name": "route_target",
            "getval": re.compile(
                r"""
                \s*route-target
                \s+(?P<action>\S+)
                \s*(?P<type>evpn|vpn-ipv4|vpn-ipv6)*
                \s*(?P<map>route-map\s\S+)*
                \s+(?P<target>\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_bgp_route_target,
            "compval": "route_target",
            "result": {
                "address_family": {
                    '{{ afi + "_" + vrf|d() }}': {
                        "route_target": {
                            "action": "{{ action }}",
                            "type": "{{ type }}",
                            "route_map": "{{ map.split(" ")[1] }}",
                            "target": "{{ target }}",
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



# Copyright (c) 2024, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:
    DOCUMENTATION = r"""
options:
  venv:
    description:
      - Use the Python interpreter from this virtual environment.
      - Pass the path to the root of the virtualenv, not the C(bin/) directory nor the C(python) executable.
    type: path
  settings:
    description:
      - Specifies the settings module to use.
      - The value is passed as is to the C(--settings) argument in C(django-admin).
    type: str
    required: true
  pythonpath:
    description:
      - Adds the given filesystem path to the Python import search path.
      - The value is passed as is to the C(--pythonpath) argument in C(django-admin).
    type: path
  traceback:
    description:
      - Provides a full stack trace in the output when a C(CommandError) is raised.
    type: bool
  verbosity:
    description:
      - Specifies the amount of notification and debug information in the output of C(django-admin).
    type: int
    choices: [0, 1, 2, 3]
  skip_checks:
    description:
      - Skips running system checks prior to running the command.
    type: bool


notes:
  - The C(django-admin) command is always executed using the C(C) locale, and the option C(--no-color) is always passed.
seealso:
  - name: django-admin and manage.py in official Django documentation
    description: >-
      Refer to this documentation for the builtin commands and options of C(django-admin). Please make sure that you select
      the right version of Django in the version selector on that page.
    link: https://docs.djangoproject.com/en/5.0/ref/django-admin/
"""

    DATABASE = r"""
options:
  database:
    description:
      - Specify the database to be used.
    type: str
    default: default
"""

    DATA = r"""
options:
  excludes:
    description:
      - Applications or models to be excluded.
      - Format must be either V(app_label) or V(app_label.ModelName).
    type: list
    elements: str
  format:
    description:
      - Serialization format of the output data.
    type: str
    default: json
    choices: [xml, json, jsonl, yaml]
notes:
  - As it is now, the module is B(not idempotent). Ensuring idempotency for this case can be a bit tricky, because it would
    amount to ensuring beforehand that all the data in the fixture file is already in the database, which is not a trivial feat.
    Unfortunately, neither C(django loaddata) nor C(django dumpdata) have a C(--dry-run) option, so the only way to know whether
    there is a change or not is to actually load or dump the data.
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Akini Ross (@akinross) <akinross@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_template
short_description: Manage Templates on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Templates on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Akini Ross (@akinross)
options:
  template:
    description:
    - The name of the template.
    type: str
    aliases: [ name ]
  template_id:
    description:
    - The id of the template.
    - This parameter is required when the O(template) needs to be updated.
    type: str
    aliases: [ id ]
  template_type:
    description:
    - The type of the template.
    - The O(template_type=application) is only intended for retrieving the Application template using O(template_id) and does not support template creation.
    type: str
    aliases: [ type ]
    choices:
      - application
      - tenant
      - l3out
      - fabric_policy
      - fabric_resource
      - monitoring_tenant
      - monitoring_access
      - service_device
  tenant:
    description:
    - The name of the tenant attached to the template.
    - Required when O(type=tenant), O(type=l3out), O(type=monitoring_tenant) or O(type=service_device).
    type: str
  sites:
    description:
    - The list of sites attached to the template.
    - Required when O(type=l3out), O(type=monitoring_tenant) or O(type=monitoring_access).
    - Set to an empty list O(sites=[]) to remove all sites attached from the template.
    - Set to null O(sites=null) or do not provide O(sites) to avoid making changes to sites attached to the template.
    aliases: [ site ]
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - The name of the site attached to the template.
        type: str
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new tenant policy template
  cisco.mso.ndo_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_policy
    template_type: tenant
    tenant: ansible_test_tenant
    sites:
      - name: ansible_test_site
    state: present

- name: Create a new l3out policy template
  cisco.mso.ndo_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_l3out_policy_template
    template_type: l3out
    tenant: ansible_test_tenant
    sites:
      - name: ansible_test_site
    state: present

- name: Query a tenant policy template
  cisco.mso.ndo_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_policy
    template_type: tenant
    state: query
  register: query_new_tenant_policy_template

- name: Query all tenant policy templates
  cisco.mso.ndo_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template_type: tenant
    state: query
  register: query_all

- name: Query all templates
  cisco.mso.ndo_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    state: query
  register: query_all

- name: Delete a tenant policy template
  cisco.mso.ndo_template:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_policy
    template_type: tenant
    state: absent
"""

RETURN = r"""
"""

import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate
from ansible_collections.cisco.mso.plugins.module_utils.constants import TEMPLATE_TYPES


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", aliases=["name"]),
        template_id=dict(type="str", aliases=["id"]),
        template_type=dict(type="str", aliases=["type"], choices=list(TEMPLATE_TYPES)),
        tenant=dict(type="str"),
        sites=dict(type="list", elements="dict", aliases=["site"], options=dict(name=dict(type="str"))),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["template", "template_type"]],
            ["state", "present", ["template", "template_type"]],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    template_id = module.params.get("template_id")
    template_type = module.params.get("template_type")
    tenant_id = mso.lookup_tenant(module.params.get("tenant"))
    site_ids = [mso.lookup_site(site.get("name")) for site in module.params.get("sites", [])] if module.params.get("sites") else []
    state = module.params.get("state")

    if state != "query" and template_type == "application":
        mso.fail_json(msg="The template_type: application is only intended for retrieving the Application template.")

    mso_template = MSOTemplate(mso, template_type, template, template_id)

    mso.existing = mso.previous = copy.deepcopy(mso_template.template)

    if state == "present":
        if tenant_id and not TEMPLATE_TYPES[template_type]["tenant"]:
            mso.fail_json(msg="Tenant cannot be attached to template of type {0}.".format(template_type))

        if not tenant_id and TEMPLATE_TYPES[template_type]["tenant"]:
            mso.fail_json(msg="Tenant must be provided for template of type {0}.".format(template_type))

        if not site_ids and TEMPLATE_TYPES[template_type]["site_amount"] == 1:
            mso.fail_json(msg="Site must be provided for template of type {0}.".format(template_type))

        if len(site_ids) > 1 and TEMPLATE_TYPES[template_type]["site_amount"] == 1:
            mso.fail_json(msg="Only one site can be attached to template of type {0}.".format(template_type))

        if mso_template.template:
            if mso_template.template.get("templateType") != TEMPLATE_TYPES[template_type]["template_type"]:
                mso.fail_json(msg="Template type cannot be changed.")

            if TEMPLATE_TYPES[template_type]["tenant"] and changed(
                mso_template.template, template_type, tenant_id, "tenantId" if TEMPLATE_TYPES[template_type] != "monitoring_acces" else "tenant"
            ):
                mso.fail_json(msg="Tenant cannot be changed.")

            if TEMPLATE_TYPES[template_type]["site_amount"] == 1 and changed(mso_template.template, template_type, site_ids[0], "siteId"):
                mso.fail_json(msg="Site cannot be changed.")

            ops = []
            if mso_template.template.get("displayName") != template:
                ops.append(dict(op="replace", path="/displayName", value=template))
                # Changing in existing object to set correctly in proposed/current output
                mso_template.template["displayName"] = template

            # When more then 1 site can be provided the sites can be added and/or removed with PATCH operations
            # This is done to avoid config loss within sites during a single replace operation on the sites container
            if TEMPLATE_TYPES[template_type]["site_amount"] > 1 and module.params.get("sites") is not None:
                append_site_config_to_ops(ops, TEMPLATE_TYPES[template_type]["template_type_container"], mso_template.template, site_ids)

            mso.sanitize(mso_template.template)

            if not module.check_mode and ops:
                mso.existing = mso.request(mso_template.template_path, method="PATCH", data=ops)

        else:
            payload = {
                "displayName": template,
                "templateType": TEMPLATE_TYPES[template_type]["template_type"],
                TEMPLATE_TYPES[template_type]["template_type_container"]: {},
            }

            # Set template specific payload in functions to increase readability due to the amount of different templates with small differences
            if template_type == "tenant":
                set_tenant_template_payload(payload, template_type, tenant_id, site_ids)
            elif template_type == "l3out":
                set_l3out_template_payload(payload, template_type, tenant_id, site_ids[0])
            elif template_type == "fabric_policy":
                set_fabric_policy_template_payload(payload, template_type, site_ids)
            elif template_type == "fabric_resource":
                set_fabric_resource_template_payload(payload, template_type, site_ids)
            elif template_type == "monitoring_tenant":
                set_monitoring_tenant_template_payload(payload, template_type, tenant_id, site_ids)
            elif template_type == "monitoring_access":
                set_monitoring_acces_template_payload(payload, template_type, site_ids)
            elif template_type == "service_device":
                set_service_device_template_payload(payload, template_type, tenant_id, site_ids)

            if not module.check_mode:
                mso.existing = mso.request(mso_template.templates_path, method="POST", data=payload)
                payload["templateId"] = mso.existing.get("templateId")

            mso.sanitize(payload)

        if module.check_mode:
            mso.existing = mso.proposed

    elif state == "absent":
        if mso.previous and not module.check_mode:
            mso.request(mso_template.template_path, method="DELETE")
        mso.existing = {}

    mso.exit_json()


def set_tenant_template_payload(payload, template_type, tenant_id, site_ids):
    payload[TEMPLATE_TYPES[template_type]["template_type_container"]].update({"template": {"tenantId": tenant_id}})
    payload[TEMPLATE_TYPES[template_type]["template_type_container"]].update({"sites": [{"siteId": site_id} for site_id in site_ids]})


def set_l3out_template_payload(payload, template_type, tenant_id, site_id):
    payload[TEMPLATE_TYPES[template_type]["template_type_container"]].update({"tenantId": tenant_id})
    payload[TEMPLATE_TYPES[template_type]["template_type_container"]].update({"siteId": site_id})


def set_fabric_policy_template_payload(payload, template_type, site_ids):
    payload[TEMPLATE_TYPES[template_type]["template_type_container"]].update({"sites": [{"siteId": site_id} for site_id in site_ids]})


def set_fabric_resource_template_payload(payload, template_type, site_ids):
    payload[TEMPLATE_TYPES[template_type]["template_type_container"]].update({"sites": [{"siteId": site_id} for site_id in site_ids]})


def set_monitoring_tenant_template_payload(payload, template_type, tenant_id, site_ids):
    payload[TEMPLATE_TYPES[template_type]["template_type_container"]].update({"template": {"mtType": "tenant", "tenant": tenant_id}})
    payload[TEMPLATE_TYPES[template_type]["template_type_container"]].update({"sites": [{"siteId": site_id} for site_id in site_ids]})


def set_monitoring_acces_template_payload(payload, template_type, site_ids):
    payload[TEMPLATE_TYPES[template_type]["template_type_container"]].update({"template": {"mtType": "access"}})
    payload[TEMPLATE_TYPES[template_type]["template_type_container"]].update({"sites": [{"siteId": site_id} for site_id in site_ids]})


def set_service_device_template_payload(payload, template_type, tenant_id, site_ids):
    payload[TEMPLATE_TYPES[template_type]["template_type_container"]].update({"template": {"tenantId": tenant_id}})
    payload[TEMPLATE_TYPES[template_type]["template_type_container"]].update({"sites": [{"siteId": site_id} for site_id in site_ids]})


def changed(config, template_type, value, key):
    if TEMPLATE_TYPES[template_type]["template_container"]:
        config_value = config.get(TEMPLATE_TYPES[template_type]["template_type_container"], {}).get("template", {}).get(key)
    else:
        config_value = config.get(TEMPLATE_TYPES[template_type]["template_type_container"], {}).get(key)
    return config_value != value


def append_site_config_to_ops(ops, template_type_container, config, site_ids):
    template_container = config.get(template_type_container, {})

    existing_site_ids = [site.get("siteId") for site in template_container.get("sites", [])]

    index_list = [index for index, site_id in enumerate(existing_site_ids) if site_id not in site_ids]

    if index_list:
        # Sort in reverse to remove from the end of the list first, this way the indexes are not shifting during removal operations
        index_list.sort(reverse=True)
        for index in index_list:
            ops.append(dict(op="remove", path="/{0}/sites/{1}".format(template_type_container, index)))
            # Changing in existing object to set correctly in proposed/current output
            template_container["sites"].pop(index)

    for site_id in [site_id for site_id in site_ids if site_id not in existing_site_ids]:
        payload = {"siteId": site_id}
        ops.append(dict(op="add", path="/{0}/sites/-".format(template_type_container), value=payload))
        # Changing in existing object to set correctly in proposed/current output
        if template_container.get("sites") is None:
            template_container["sites"] = []
        template_container["sites"].append(payload)


if __name__ == "__main__":
    main()



# Copyright (c) 2012, Jan-Piet Mens <jpmens(at)gmail.com>
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
name: dnstxt
author: Jan-Piet Mens (@jpmens) <jpmens(at)gmail.com>
short_description: Query a domain(s)'s DNS txt fields
requirements:
  - dns/dns.resolver (python library)
description:
  - Uses a python library to return the DNS TXT record for a domain.
options:
  _terms:
    description: Domain or list of domains to query TXT records from.
    required: true
    type: list
    elements: string
  real_empty:
    description:
      - Return empty result without empty strings, and return empty list instead of V(NXDOMAIN).
      - The default for this option is likely to change to V(true) in the future.
    default: false
    type: bool
    version_added: 6.0.0
"""

EXAMPLES = r"""
- name: show txt entry
  ansible.builtin.debug:
    msg: "{{lookup('community.general.dnstxt', ['test.example.com'])}}"

- name: iterate over txt entries
  ansible.builtin.debug:
    msg: "{{item}}"
  with_community.general.dnstxt:
    - 'test.example.com'
    - 'other.example.com'
    - 'last.example.com'

- name: iterate of a comma delimited DNS TXT entry
  ansible.builtin.debug:
    msg: "{{item}}"
  with_community.general.dnstxt: "{{lookup('community.general.dnstxt', ['test.example.com']).split(',')}}"
"""

RETURN = r"""
_list:
  description:
    - Values returned by the DNS TXT record.
  type: list
"""

HAVE_DNS = False
try:
    import dns.resolver
    from dns.exception import DNSException

    HAVE_DNS = True
except ImportError:
    pass

from ansible.errors import AnsibleError
from ansible.plugins.lookup import LookupBase

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms

# ==============================================================
# DNSTXT: DNS TXT records
#
#       key=domainname
# TODO: configurable resolver IPs
# --------------------------------------------------------------


class LookupModule(LookupBase):
    def run(self, terms, variables=None, **kwargs):
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)

        if HAVE_DNS is False:
            raise AnsibleError("Can't LOOKUP(dnstxt): module dns.resolver is not installed")

        real_empty = self.get_option("real_empty")

        ret = []
        for term in terms:
            domain = term.split()[0]
            string = []
            try:
                answers = dns.resolver.query(domain, "TXT")
                for rdata in answers:
                    s = rdata.to_text()
                    string.append(s[1:-1])  # Strip outside quotes on TXT rdata

            except dns.resolver.NXDOMAIN:
                if real_empty:
                    continue
                string = "NXDOMAIN"
            except dns.resolver.Timeout:
                if real_empty:
                    continue
                string = ""
            except dns.resolver.NoAnswer:
                if real_empty:
                    continue
                string = ""
            except DNSException as e:
                raise AnsibleError(f"dns.resolver unhandled exception {e}") from e

            ret.append("".join(string))

        return ret



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_appliance_sdwan_internet_policies
short_description: Resource module for networks _appliance _sdwan _internet _policies
description:
  - Manage operation update of the resource networks _appliance _sdwan _internet _policies.
  - Update SDWAN internet traffic preferences for an MX network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  wanTrafficUplinkPreferences:
    description: Policies with respective traffic filters for an MX network.
    elements: dict
    suboptions:
      failOverCriterion:
        description: WAN failover and failback behavior.
        type: str
      performanceClass:
        description: Performance class setting for uplink preference rule.
        suboptions:
          builtinPerformanceClassName:
            description: Name of builtin performance class. Must be present when performanceClass
              type is 'builtin' and value must be one of 'VoIP'.
            type: str
          customPerformanceClassId:
            description: ID of created custom performance class, must be present when
              performanceClass type is "custom".
            type: str
          type:
            description: Type of this performance class. Must be one of 'builtin'
              or 'custom'.
            type: str
        type: dict
      preferredUplink:
        description: Preferred uplink for uplink preference rule. Must be one of 'wan1',
          'wan2', 'bestForVoIP', 'loadBalancing', 'defaultUplink' or 'cellular'.
        type: str
      trafficFilters:
        description: Traffic filters.
        elements: dict
        suboptions:
          type:
            description: Traffic filter type. Must be 'custom', 'major_application',
              'application (NBAR)', if type is 'application', you can pass either
              an NBAR App Category or Application.
            type: str
          value:
            description: Value of traffic filter.
            suboptions:
              destination:
                description: Destination of 'custom' type traffic filter.
                suboptions:
                  applications:
                    description: List of application objects (either majorApplication
                      or nbar).
                    elements: dict
                    suboptions:
                      id:
                        description: Id of the major application, or a list of NBAR
                          Application Category or Application selections.
                        type: str
                      name:
                        description: Name of the major application or application
                          category selected.
                        type: str
                      type:
                        description: App type (major or nbar).
                        type: str
                    type: list
                  cidr:
                    description: CIDR format address (e.g."192.168.10.1", which is
                      the same as "192.168.10.1/32"), or "any".
                    type: str
                  port:
                    description: E.g. "any", "0" (also means "any"), "8080", "1-1024".
                    type: str
                type: dict
              protocol:
                description: Protocol of the traffic filter. Must be one of 'tcp',
                  'udp', 'icmp6' or 'any'.
                type: str
              source:
                description: Source of traffic filter.
                suboptions:
                  cidr:
                    description: CIDR format address (e.g."192.168.10.1", which is
                      the same as "192.168.10.1/32"), or "any". Cannot be used in
                      combination with the "vlan" property.
                    type: str
                  host:
                    description: Host ID in the VLAN. Should not exceed the VLAN subnet
                      capacity. Must be used along with the "vlan" property and is
                      currently only available under a template network.
                    type: int
                  port:
                    description: E.g. "any", "0" (also means "any"), "8080", "1-1024".
                    type: str
                  vlan:
                    description: VLAN ID of the configured VLAN in the Meraki network.
                      Cannot be used in combination with the "cidr" property and is
                      currently only available under a template network.
                    type: int
                type: dict
            type: dict
        type: list
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance updateNetworkApplianceSdwanInternetPolicies
    description: Complete reference of the updateNetworkApplianceSdwanInternetPolicies
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-sdwan-internet-policies
notes:
  - SDK Method used are
    appliance.Appliance.update_network_appliance_sdwan_internet_policies,
  - Paths used are
    put /networks/{networkId}/appliance/sdwan/internetPolicies,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_appliance_sdwan_internet_policies:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkId: string
    wanTrafficUplinkPreferences:
      - failOverCriterion: poorPerformance
        performanceClass:
          builtinPerformanceClassName: VoIP
          customPerformanceClassId: '123456'
          type: custom
        preferredUplink: wan1
        trafficFilters:
          - type: custom
            value:
              destination:
                applications:
                  - id: meraki:layer7/application/3
                    name: DNS
                    type: major
                cidr: any
                port: any
              protocol: tcp
              source:
                cidr: 192.168.1.0/24
                host: 254
                port: 1-1024
                vlan: 10
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "wanTrafficUplinkPreferences": [
        {
          "preferredUplink": "string",
          "failOverCriterion": "string",
          "performanceClass": {
            "type": "string",
            "builtinPerformanceClassName": "string",
            "customPerformanceClassId": "string"
          },
          "trafficFilters": [
            {
              "type": "string",
              "value": {
                "protocol": "string",
                "source": {
                  "port": "string",
                  "cidr": "string",
                  "vlan": 0,
                  "host": 0
                },
                "destination": {
                  "port": "string",
                  "cidr": "string",
                  "applications": [
                    {
                      "id": "string",
                      "name": "string",
                      "type": "string"
                    }
                  ]
                }
              }
            }
          ]
        }
      ]
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_set_vpn_community_remote_access
short_description: Edit existing Remote Access object. Using object name or uid is optional.
description:
  - Edit existing Remote Access object. Using object name or uid is optional.
  - Add and Delete API commands for this object are unavailable since there is single object per domain.
  - All operations are performed over Web Services API.
  - Available from R80.40 JHF management version.
version_added: "5.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name.
    type: str
  gateways:
    description:
      - Collection of Gateway objects identified by the name or UID.
    type: list
    elements: str
  user_groups:
    description:
      - Collection of User group objects identified by the name or UID.
    type: list
    elements: str
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  override_vpn_domains:
    description:
      - The Overrides VPN Domains of the participants GWs.
      - Available from R82 JHF management version.
    type: list
    elements: dict
    version_added: "6.5.0"
    suboptions:
      gateway:
        description:
          - Participant gateway in override VPN domain identified by the name or UID.
        type: str
      vpn_domain:
        description:
          - <html>VPN domain network<br><b>Relevant only in Domain-Based VPN Communities</b></html> identified by the name or UID.
        type: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: set-vpn-community-remote-access
  cp_mgmt_set_vpn_community_remote_access:
    gateways:
      - mygateway
    user_groups:
      - myusergroup
"""

RETURN = """
cp_mgmt_set_vpn_community_remote_access:
  description: The checkpoint set-vpn-community-remote-access output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        name=dict(type='str'),
        gateways=dict(type='list', elements='str'),
        user_groups=dict(type='list', elements='str'),
        tags=dict(type='list', elements='str'),
        override_vpn_domains=dict(type='list', elements='dict', options=dict(gateway=dict(type='str'), vpn_domain=dict(type='str'))),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "set-vpn-community-remote-access"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()
